// 
// Simple DDE client interface for PDICW servers.
//   Copyright (C) 2001 Ichiro AKAI.
//
// Compile on Visual C++ Ver.6 
// 
// This software is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with GNU Emacs; see the file COPYING.  If not, write to
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
// 

#include "stdafx.h"
#include <windows.h>
#include <ddeml.h>
#include <process.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define MAXSZBUFF 1024
#define PDICWSERVER   "PDICW"
#define PDICWTOPIC    "Dictionary"
#define SLEEPLOOPMAX  50

char szBuff[MAXSZBUFF];
char szSep[] = " \t\r";
	STARTUPINFO si;
	PROCESS_INFORMATION pi;

//
// Call back function
//
HDDEDATA CALLBACK 
DdeCallback (UINT uType, UINT uFmt, HCONV hconv,
	     HSZ hsz1, HSZ hsz2, HDDEDATA hdata,
	     DWORD dwData1, DWORD dwData2){
  return ((HDDEDATA) NULL);
}

//
// Poke Transaction error report
//
void error_poke( char* cmd, char *data )
{
  fprintf( stdout, "Error : POKE [%s] [%s]\n", cmd, data );
  fflush( stdout );
}

//
// Poke Transaction
//
int DdePoke( DWORD idDde, char *Item, char *Data, HCONV hConv, int err_report )
{
	HSZ hszItem = DdeCreateStringHandle( idDde, Item, CP_WINANSI );
	if( !hszItem ) return 1;

	HDDEDATA hszDat = DdeCreateDataHandle( 
                        idDde, 
                        (unsigned char *)Data, 
                        strlen(Data)+ 1,
		                    0, hszItem, CF_TEXT, 0);
  HDDEDATA hdata;

	if( hszDat ){
		hdata = DdeClientTransaction( 
                        (LPBYTE)hszDat, 
                        (DWORD)-1, 
                        hConv, hszItem,
                        CF_TEXT, XTYP_POKE, 30000, NULL);	// [h(10b҂)

  	DdeFreeDataHandle ( hdata );
	}
	DdeFreeDataHandle ( hszDat );
	DdeFreeStringHandle ( idDde, hszItem );

//  printf( "[%s] (%s)->[%d]\n", Item, Data, (int)hdata);
  if((err_report == 1) && ( hdata == 0 )) error_poke( Item, Data );

	return (int)hdata;
}

//
// Request(Peek) Transaction error report
//
void error_request( char* cmd, char *data )
{
  fprintf( stdout, "Error : REQUEST [%s] [%s]\n", cmd, data );
  fflush( stdout );
}

//
// Request(Peek) Transaction
//
int DdeRequest( DWORD idDde, char *Item, char *Data, HCONV hConv )
{
	int r;

	HSZ hszItem = DdeCreateStringHandle( idDde, Item, CP_WINANSI );
	if( !hszItem ) return 1;

	HDDEDATA hdata = DdeClientTransaction( 
                        (LPBYTE)NULL, 
                        (DWORD)-1, 
                        hConv, hszItem,
                        CF_TEXT, XTYP_REQUEST, 30000, NULL);	// [h(10b҂)

  if( hdata != 0 ){
		DWORD size = DdeGetData(hdata, NULL, 0, 0);
    if( size > 0 ){
      unsigned char *data = (unsigned char *)malloc( size+2 );
      if( NULL != data ){
        DdeGetData(hdata, data, size+1, 0);
        fprintf(stdout, "%s", data);
        fflush(stdout);
        r = 0;
      }
      free( data );
    }
  }
	DdeFreeStringHandle ( idDde, hszItem);

  if( hdata == 0 ) error_request( Item, Data );
  return (int)hdata;
}

int DDEServerStart( char *cmdline )
{

  HANDLE hFile = CreateFile( cmdline, GENERIC_READ, FILE_SHARE_READ,
                             NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL,
                             NULL);
  if( hFile == INVALID_HANDLE_VALUE ){
    return 1;
  }
  else{
    CloseHandle(hFile);
  }

  GetStartupInfo(&si);

  if( ! CreateProcess( NULL, cmdline, NULL, NULL, FALSE, 
                       CREATE_NEW_CONSOLE, NULL, NULL, &si, &pi ) ){
		return 2;
	}
	return 0;
}

// 
// main function
//
int main (int argc, char *argv[])
{
  int rtn = 0;
  DWORD idDde = 0;
  HCONV hConv;
  HSZ   Server;
  HSZ   Topic = 0;
  char  *p, *cmd, *data;
  int   i, len;

  // DDE Initialize
  if( DMLERR_NO_ERROR != DdeInitialize( 
                              &idDde, 
                              (PFNCALLBACK)DdeCallback, 
                              APPCMD_CLIENTONLY, 0) ){
    fprintf (stdout, "Error : DDE Initialize\n");
    fflush( stdout );
    exit (1);
  }

  // create Server and Topic handle
  Server = DdeCreateStringHandle( idDde, PDICWSERVER, CP_WINANSI );
  Topic  = DdeCreateStringHandle( idDde, PDICWTOPIC,  CP_WINANSI );
  if( ( Server == 0L )||( Topic == 0L ) ){
    fprintf (stdout, "Error : PDICW DDE Server or Topic\n");
    fflush( stdout );
    exit (1);
  }

  // DDE connection
  hConv = DdeConnect( idDde, Server, Topic, NULL);
  if( hConv == 0 ){
    if( argc > 1 ){
      if( strlen( argv[1] ) ){

        int loop = 50;

        // Boot up PDICW32.exe
        switch( DDEServerStart(argv[1]) ){
          case 1:
            fprintf (stdout, "Error : No File of %s.\n", argv[1]);
            fflush( stdout );
            rtn = 1;
            goto Termination;
            break;
          case 2:
            fprintf (stdout, "Error : CreateProcess of %s.\n", argv[1]);
            fflush( stdout );
            rtn = 1;
            goto Termination;
            break;
          default:
            break;
        }

        // waiting loop for DDE connection
        for( loop = 0 ; loop < SLEEPLOOPMAX ; loop++ ){
          SleepEx((DWORD)200,FALSE);
          hConv = DdeConnect( idDde, Server, Topic, NULL);
          if( hConv != 0 ){
            break;
          }
        }
      }
    }
  }

  // connected ?
  if( hConv == 0 ){
    fprintf (stdout, "Error : Connect\n");
    fflush( stdout );
    rtn = 1;
    goto Termination;
  }

  // input from stdout
  while( NULL != fgets( szBuff, MAXSZBUFF, stdin ) ){
    if( NULL != ( p = strrchr( szBuff, '\n' ) ) ) *p = (char)0;

    // printf("[%s]\n", szBuff);
    cmd = (char *)szBuff;
    // scaning space or tab char at head
    while( *cmd != (char)0 ){
      if( NULL == strchr( " \t", (int)(*cmd) ) )  break;
      cmd++;
    }

    // scaning the end of command
    if( strlen(cmd) > 0 ){
      data = cmd;
      while( *data != (char)0 ){
        if( NULL != strchr( " \t", (int)(*data) ) ){
          *data = (char)0;
          data++;
          break;
        }
        data++;
      }
    }

    // scaning space or tab char at head
    while( *data != (char)0 ){
      if( NULL == strchr( " \t", (int)(*data) ) )  break;
      data++;
    }

    // scaning space or tab char at the end
    len = strlen(data);
    if( len > 0 ){
      for( i = len - 1; i >=0 ; i-- ){
        if( NULL != strchr( " \t", (int)data[i] ) ){
          data[i] = (char)0;
        }
        else{
          break;
        }
      }
    }

    // cmd upper string 
	  _strupr( cmd );

    if( strcmp( cmd, "OPEN" ) == 0 ){
      if( 0 == DdePoke( idDde, cmd, data, hConv,1 ) ) goto Disconnect;	
    }

    // command only with poke
    if( ( strcmp( cmd, "CLOSE" ) == 0 )||( strcmp( cmd, "CONFIG" ) == 0 )||
        ( strcmp( cmd, "FORMAT" ) == 0 )||( strcmp( cmd, "SELECT" ) == 0 )||
        ( strcmp( cmd, "POPUPSEARCH" ) == 0 ) ){
      DdePoke( idDde, cmd, data, hConv,1 );
    }

    // command with looping poke and request
    if( ( strcmp( cmd, "FIND" ) == 0 )||( strcmp( cmd, "FORWARDSEARCH" ) == 0 )){
      if( 1 != DdePoke( idDde, cmd, data, hConv, 0 ) ){
        fprintf( stdout, "NOT FOUND \"%s\"\n", data );
      }
      else{
        DdeRequest( idDde, cmd, "", hConv );
        while( 1 == DdePoke( idDde, cmd, "", hConv, 0 ) ){
          DdeRequest( idDde, cmd, "", hConv );
        }
      }
    }

    // search command does not work with looping poke and request
    // only onece.
    if( strcmp( cmd, "SEARCH" ) == 0 ){
      if( 1 != DdePoke( idDde, cmd, data, hConv, 0 ) ){
        fprintf( stdout, "NOT FOUND \"%s\"\n", data );
      }
      else{
        DdeRequest( idDde, cmd, "", hConv );
      }
    }

	  if( ( strcmp( cmd, "QUIT" ) == 0 )||( strcmp( cmd, "EXIT" ) == 0 )){
	  	break;
	  }
  }

Disconnect:
  DdeDisconnect (hConv);

Termination:
  DdeFreeStringHandle (idDde, Server);
  if(Topic)  DdeFreeStringHandle (idDde, Topic);

  DdeUninitialize (idDde);

  return (rtn);
}

