;;; pdicw.el --- Personal Dictionary for Windows (PDICW32)
;;;                 Version 2.01, 9 Aug 2001.

;; Copyright (C) 2001 Ichiro AKAI (iakai@a5w.highway.ne.jp)

;; Keywords: PDICW, DDE, w32, MS-Windows

;; Front End Emacs Lisp Functions for Personal Dictionary for Win32
;; (PDICW32) on MS-Windows. 

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(defcustom pdicw-command "\"c:\\usr\\PDICW\\PDICW32.exe\""
  "*PDICW: The program of Personal Dictionary for Win32 (PDICW).")

(defconst pdicw-server "PDICW"
  "PDICW: The server name of Personal Dictionary for Win32 (PDICW).")

(defconst pdicw-topic "Dictionary"
  "PDICW: The Topic name of Personal Dictionary for Win32 (PDICW).")

(defconst pdicw-output-buffer-name "*PDICW*"
  "PDICW: The name of buffer for PDICW.")

(defconst pdicw-command-buffer-name "*PDICW-Command*"
  "PDICW: The name of command buffer for PDICW.")

(defcustom pdicw-shell "cmdproxy.exe";shell-file-name 
  "PDICW: shell for PDICW")

(defcustom pdicw-shell-switch 
  (cond ((boundp 'shell-command-option)
	 shell-command-option)
	((boundp 'shell-command-switch)
	 shell-command-switch)
	(t "/c"))
  "PDICW: shell switch for PDICW")

(defcustom pdicw-result-window-height 10
  "PDICW: Height of thw window for result report")

(defun pdicw-kill-output-buffer ()
  "pdicw-kill-output-buffer"
  (if (get-buffer pdicw-output-buffer-name)
	(kill-buffer pdicw-output-buffer-name)))

(defun pdicw-kill-command-buffer ()
  "pdicw-kill-command-buffer"
  (if (get-buffer pdicw-command-buffer-name)
	(kill-buffer pdicw-command-buffer-name)))

(defun pdicw-bootup ()
  "pdicw-bootup"
  (message nil)
  (princ (concat "Starting up " 
		 pdicw-command 
		 "...") t)
  (call-process pdicw-shell nil 0 nil 
		pdicw-shell-switch 
		pdicw-command)
  (princ "done." t)
  (message nil))

(defun pdicw-call-ddeproxy ();(command-buff, output-buff)
  "Calling ddeproxy"
  (pdicw-kill-output-buffer)
  (set-buffer pdicw-command-buffer-name)
  (call-process-region (point-min) (point-max) 
		       pdicw-shell nil 
		       pdicw-output-buffer-name nil pdicw-shell-switch
		       (concat "pdicwdde " pdicw-command)))

(defun pdicw-is-error( error )
  "Error check"
  (if (get-buffer pdicw-output-buffer-name)
      (progn 
	(set-buffer pdicw-output-buffer-name)
	(goto-char (point-min))
	(if (search-forward error (point-max) t)
	    t
	  nil))
    t))
  
(defun pdicw-popup-buffer (word)
  "Filter for DDE proxy output."
  (let ( (cur-buffer (current-buffer)) win-height )
    (save-excursion
      (set-buffer pdicw-output-buffer-name)
      (goto-char (point-min))
      (replace-string "\r" "") 
      (goto-char (point-min))
      (replace-string "/" "/\n") 
      (goto-char (point-min))
      (replace-string " y" "y") 
      (goto-char (point-min))
      (replace-string " : " "\n\t: ") 
      (goto-char (point-min))
      (setq buffer-read-only t))
    (pop-to-buffer pdicw-output-buffer-name)
    (setq win-height (window-height))
    (shrink-window (- win-height pdicw-result-window-height))
    (switch-to-buffer-other-window cur-buffer)
    (message (concat "\"" word "\", PDICW search was completed."))))

(defun pdicw-call ();(command-buff, output-buff)
  "Calling ddeproxy"
  (interactive)
  (pdicw-call-ddeproxy)
  (let ((rtn-code nil))
    (cond
     ((pdicw-is-error "pdicwdde: not found")
      (message "Error : pdicwdde does not exists in the PATH."))
     ((or (pdicw-is-error "Error : DDE Initialize")
	  (pdicw-is-error "Error : PDICW DDE Server or Topic"))
      (message "Error : DDE Connection Initialize"))
     ((pdicw-is-error "Error : No File of")
      (message (concat "Error : " pdicw-command " does not exist.")))
     ((pdicw-is-error "Error : CreateProcess of")
      (message (concat "Error : Booting of " pdicw-command)))
     ((pdicw-is-error "Error : Connect")
      (message (concat "Error : DDE connection fault on " pdicw-command)))
     ((or (pdicw-is-error "Error : POKE")
	  (pdicw-is-error "Error : REQUEST"))
      (message "Error : DDE POKE or REQUEST"))
     (t
      (setq rtn-code t)))
    rtn-code))


(defun pdicw-popup-search-word (word)
  "PDICW: PopupSearch a word."
  (interactive "sPopupSearch Word >")
  (save-excursion
    (pdicw-kill-command-buffer)
    (generate-new-buffer pdicw-command-buffer-name)
    (set-buffer pdicw-command-buffer-name)
    (insert-before-markers "open\n")
    (insert-before-markers (concat "popupsearch " (pdicw-format-word word) "\n"))
    (insert-before-markers "close\n")
    (insert-before-markers "quit\n\n")
    (pdicw-call)))

(defun pdicw-search-method-word (method word)
  "PDICW: PopupSearch a word."
  (let ((status))
    (save-excursion
      (pdicw-kill-command-buffer)
      (generate-new-buffer pdicw-command-buffer-name)
      (set-buffer pdicw-command-buffer-name)
      (insert-before-markers "open\n")
      (insert-before-markers (concat method " " word "\n"))
      (insert-before-markers "close\n")
      (insert-before-markers "quit\n\n")
      (setq status (pdicw-call)))
    (if status
	 (pdicw-popup-buffer word))))

(defun pdicw-find-word (word)
  "PDICW: Find a word."
  (interactive "sFind Word >")
  (pdicw-search-method-word "find" (pdicw-format-word word)))

(defun pdicw-search-word (word)
  "PDICW: Search a word."
  (interactive "sSearch Word >")
  (pdicw-search-method-word "search" (pdicw-format-word word)))
  
(defun pdicw-forwardsearch-word (word)
  "PDICW: ForwardSearch a word."
  (interactive "sForwardSearch Word >")
  (pdicw-search-method-word "forwardsearch" (pdicw-format-word word)))
  
(defun pdicw-get-word-at-point-simple ()
  "PDICW: Get a word at point by simple."
  (if (fboundp 'thing-at-point)
      (thing-at-point 'word)
    (buffer-substring-no-properties
     (progn (skip-syntax-backward "w") (point))
     (progn (skip-syntax-forward "w") (point)))))

(defvar pdicw-word-hist-list nil )

(defvar pdicw-get-word-at-point 'pdicw-get-word-at-point-simple)

(defun pdicw-format-word (word)
  "PDICW: reconstruct the word."
  (let ((start 0) (real-word ""))
    (while (string-match "\\(\\w+\\)" word start)
      (setq real-word 
	    (concat real-word 
		    (if (> (length real-word) 0) " " "")
		    (match-string 0 word)))
      (setq start (match-end 0)))
    real-word))

(defun pdicw-input-word (method)
  "PDICW: Get a word at point and input the word."
  (let ( (word (funcall pdicw-get-word-at-point)) )
    (read-from-minibuffer (concat method " Word : ") 
			  word nil nil 
			  'pdicw-word-hist-list)))

(defun pdicw-popup-search-word-at-point ()
  "PDICW: PopupSearch a word at point."
  (interactive)
  (pdicw-popup-search-word (pdicw-input-word "PopupSearch")))

(defun pdicw-find-word-at-point ()
  "PDICW: Find a word at point."
  (interactive)
  (pdicw-find-word (pdicw-input-word "Find")))

(defun pdicw-search-word-at-point ()
  "PDICW: Search a word at point."
  (interactive)
  (pdicw-search-word (pdicw-input-word "Search")))

(defun pdicw-forwardsearch-word-at-point ()
  "PDICW: ForwardSearch a word at point."
  (interactive)
  (pdicw-forwardsearch-word (pdicw-input-word "ForwardSearch")))

(defun pdicw-input-region-word (method start end)
  "PDICW: Get a word at point and input the word."
  (let ( (word (buffer-substring-no-properties start end)) )
    (setq word (pdicw-format-word word))
    (read-from-minibuffer (concat method " Word : ") 
			  word nil nil 
			  'pdicw-word-hist-list)))

(defun pdicw-popup-search-word-region (start end)
  "PDICW: PopupSearch a word in region."
  (interactive "r")
  (pdicw-popup-search-word (pdicw-input-region-word "PopupSearch" 
						    start end)))

(defun pdicw-find-word-region (start end)
  "PDICW: Find a word in region."
  (interactive "r")
  (pdicw-find-word (pdicw-input-region-word "Find" 
					    start end)))

(defun pdicw-search-word-region (start end)
  "PDICW: Search a word in region."
  (interactive "r")
  (pdicw-search-word (pdicw-input-region-word "Search" 
					      start end)))

(defun pdicw-forwardsearch-word-region (start end)
  "PDICW: ForwardSearch a word in region."
  (interactive "r")
  (pdicw-forwardsearch-word (pdicw-input-region-word "ForwardSearch" 
						     start end)))


;;; PDICW Global menu definition.

(setq menu-bar-pdicw-menu (make-sparse-keymap "pdicw"))

(define-key menu-bar-pdicw-menu [pdicw-forwardsearch-word] 
  '("ForwardSearch the word" . pdicw-forwardsearch-word))

(define-key menu-bar-pdicw-menu [pdicw-search-word] 
  '("Search the word" . pdicw-search-word))

(define-key menu-bar-pdicw-menu [pdicw-find-word] 
  '("Find the word" . pdicw-find-word))

(define-key menu-bar-pdicw-menu [pdicw-popup-search-word] 
  '("PopupSearch the word" . pdicw-popup-search-word))

(define-key menu-bar-pdicw-menu [pdicw-dev2] '("" . null))

(define-key menu-bar-pdicw-menu [pdicw-forwardsearch-word-region] 
  '("ForwardSearch in region" . pdicw-forwardsearch-word-region))

(define-key menu-bar-pdicw-menu [pdicw-search-word-region] 
  '("Search in region" . pdicw-search-word-region))

(define-key menu-bar-pdicw-menu [pdicw-find-word-region] 
  '("Find in region" . pdicw-find-word-region))

(define-key menu-bar-pdicw-menu [pdicw-popup-search-word-region] 
  '("PopupSearch in region" . pdicw-popup-search-word-region))

(define-key menu-bar-pdicw-menu [pdicw-dev1] '("" . null))

(define-key menu-bar-pdicw-menu [pdicw-forwardsearch-word-at-point] 
  '("ForwardSearch at point" . pdicw-forwardsearch-word-at-point))

(define-key menu-bar-pdicw-menu [pdicw-search-word-at-point] 
  '("Search at point" . pdicw-search-word-at-point))

(define-key menu-bar-pdicw-menu [pdicw-find-word-at-point] 
  '("Find at point" . pdicw-find-word-at-point))

(define-key menu-bar-pdicw-menu [pdicw-popup-search-word-at-point] 
  '("PopupSearch at point" . pdicw-popup-search-word-at-point))

;(define-key global-map [menu-bar pdicw]
;  (cons "PDICW" menu-bar-pdicw-menu))

;(setq menu-bar-final-items (cons 'pdicw menu-bar-final-items))

(if (not (boundp 'menu-bar-dict-menu))
    (progn
      (setq menu-bar-dict-menu (make-sparse-keymap "Dictionary"))
      (define-key global-map [menu-bar dict]
	(cons "Dictionary" menu-bar-dict-menu))
      (setq menu-bar-final-items (cons 'dict menu-bar-final-items))))

(define-key menu-bar-dict-menu [pdicw]
  (cons "PDICW" menu-bar-pdicw-menu))

